#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "ImageLibs\\ximage.h"
#include "pdf417_dham.c"

typedef unsigned int UInt32;
typedef int Int32;
typedef unsigned char bit;
typedef __int64 UInt64;

int mask[15];
int dump = 0;
int encfmt = 1;          // 0
int numouts = 0;
int numerasures = 0;
int codewords[34*90];  /* array for the extracted codewords */
int erasures[34*90];   /* for the Reed-Solomon correction routine */
int Alpha_to[1024];
int Index_of[1024];

void decode_segment(int *cw, int len, int mode);
void convert_byte(int *cw, int len, int mode);
void convert_text(int *cw, int len);
void convert_num(int *cw, int len);

#define PBM_BLACK 1
#define NN      1024
#define PRIM    1
#define GPRIME  929
#define KK      32
#define A0      928
#define Ldec    1
#define FALSE   0
#define TRUE    1

static int rs_init = FALSE;

// initialize table of 3**i for syndrome calculation
void powers_init()
{
  int ii, power_of_3=1;
  Index_of[1] = GPRIME - 1;
  for (ii = 0; ii < GPRIME - 1; ii += 1) {
    Alpha_to[ii] = power_of_3;
    if (power_of_3 < GPRIME) if(ii != GPRIME-1) Index_of[power_of_3]=ii;
    else  printf("Internal error: powers of 3 calculation\n");
    power_of_3 = (power_of_3 * 3) % GPRIME;
  }
  Index_of[0] = GPRIME-1;
  Alpha_to[GPRIME-1] = 1;
  Index_of[GPRIME] = A0;
}

#define modbase(x) ((x) % (GPRIME - 1))
/* Performs ERRORS+ERASURES decoding of RS codes. If decoding is successful,
 * writes the codeword into data[] itself. Otherwise data[] is unaltered.
 * Return number of symbols corrected, or -1 if codeword is illegal
 * or uncorrectable. If eras_pos is non-null, the detected error locations
 * are written back. NOTE! This array must be at least NN-KK elements long.
 * First "no_eras" erasures are declared by the calling program. Then, the
 * maximum # of errors correctable is t_after_eras = floor((NN-KK-no_eras)/2).
 * If the number of channel errors is not greater than "t_after_eras" the
 * transmitted codeword will be recovered. Details of algorithm can be found
 * in R. Blahut's "Theory ... of Error-Correcting Codes".
 * Warning: the eras_pos[] array must not contain duplicate entries; decoder failure
 * will result. The decoder *could* check for this condition, but it would involve
 * extra time on every decoding operation.*/
int eras_dec_rs(int data[NN], int eras_pos[NN-KK], int no_eras,
                int data_len, int synd_len)
{
  int deg_lambda, el, deg_omega;
  int i, j, r, k;
  int u, q, tmp, num1, num2, den, discr_r;
  int lambda[2048+1], s[2048+1];  /* Err+Eras Locator poly and syndrome poly */
  int b[2048+1], t[2048+1], omega[2048+1];
  int root[2048], reg[2048+1], loc[2048];
  int syn_error, count;
  int ci;
  int error_val;
  int fix_loc;

  if(!rs_init) powers_init();          /* Check for illegal input values */
  for(i=0; i<data_len; i++) if (data[i] > GPRIME) return -1;
             /* form the syndromes; i.e. evaluate data(x) at roots of g(x)
                            namely @**(1+i)*PRIM, i = 0, ... , (NN-KK-1) */
  for (i = 1; i <= synd_len; i++) s[i] = 0;           //data[data_len];
  for (j = 1; j <= data_len; j++) {
    if (data[data_len - j] == 0) continue;
    tmp = Index_of[data[data_len - j]];
                           /*  s[i] ^= Alpha_to[modbase(tmp + (1+i-1)*j)]; */
    for (i = 1; i <= synd_len; i++)
      s[i] = (s[i] + Alpha_to[modbase(tmp + (i) * j)]) % GPRIME;
  }     /* Convert syndromes to index form, checking for nonzero condition */
  syn_error = 0;
  for (i=1; i <= synd_len; i++) { syn_error |=s[i]; s[i]=Index_of[s[i]]; }
  if (!syn_error) {
      /* if syndrome is zero, data[] is a codeword and there are no
       * errors to correct. So return data[] unmodified       */
    count = 0;
    printf("No errors \n");
    goto finish;
  }
  for (ci=synd_len-1; ci>=0; ci--)lambda[ci+1]=0;
  lambda[0] = 1;
  if (no_eras > 0) {
                      /* Init lambda to be the erasure locator polynomial */
    lambda[1] = Alpha_to[modbase(PRIM * eras_pos[0])];
    for (i = 1; i < no_eras; i++) {
      u = modbase(PRIM * eras_pos[i]);
      for (j = i + 1; j > 0; j--) {
        tmp = Index_of[lambda[j - 1]];
        if (tmp != A0)
            lambda[j] = (lambda[j] + Alpha_to[modbase(u + tmp)]) % GPRIME;
	  }
    }              /* Test code that verifies the erasure locator polynomial
                      just constructed,Needed only for decoder debugging. */
                         /* find roots of the erasure location polynomial */
    for (i = 1; i <= no_eras; i++)  reg[i] = Index_of[lambda[i]];
    count = 0;              // usg NN = GPRIME-1?
    for(i=1, k=data_len-Ldec; i<=data_len+synd_len;
         i++, k=modbase(data_len+k-Ldec)) {
	  q = 1;
      for (j = 1; j <= no_eras; j++) if (reg[j] != A0) {
        reg[j] = modbase(reg[j] + j);
    	q = (q + Alpha_to[reg[j]]) % GPRIME;
      }
      if (q != 0)  continue;
        /* store root and error location number indices */
      root[count] = i;
      loc[count] = k;
      count++;
    }
    if (count != no_eras) {
          // printf("\n lambda(x) is WRONG\n"); count=-1; goto finish;
    }
  }
    for (i = 0; i < synd_len + 1; i++) b[i] = Index_of[lambda[i]];
    /* Begin Berlekamp-Massey algorithm to determine error+erasure
     * locator polynomial */
    r = no_eras;
    el = no_eras;
    while (++r <= synd_len) {   /* r is the step number */
        /* Compute discrepancy at the r-th step in poly-form */
      discr_r = 0;
      for (i = 0; i < r; i++) {
        if ((lambda[i] != 0) && (s[r - i] != A0)) {
          if (i % 2 == 1) {
            discr_r = (discr_r + Alpha_to[modbase((Index_of[lambda[i]]
                        + s[r - i]))]) % GPRIME;
          } else {
            discr_r = (discr_r + GPRIME - Alpha_to[modbase((Index_of[lambda[i]]
                        + s[r - i]))]) % GPRIME;
          }
        }
      }
      discr_r = Index_of[discr_r];    /* Index form */
      if (discr_r == A0) {
          /* 2 lines below: B(x) <-- x*B(x)  COPYDOWN(&b[1],b,synd_len);  */
        for (ci = synd_len - 1; ci >= 0; ci--) b[ci + 1] = b[ci];
        b[0] = A0;
      } else {
          /* 7 lines below: T(x) <-- lambda(x) - discr_r*x*b(x) */
          /*  the T(x) will become the next lambda */
        t[0] = lambda[0];
        for (i = 0; i < synd_len; i++) {
          if (b[i] != A0) {
             t[i + 1] = (lambda[i + 1] + Alpha_to[modbase(discr_r + b[i])])
                   % GPRIME;
          } else  t[i + 1] = lambda[i + 1];
        }
        el = 0;
        if (2 * el <= r + no_eras - 1) {
          el = r + no_eras - el;
                     /* 2 lines below: B(x) <-- inv(discr_r) * lambda(x) */
          for (i = 0; i <= synd_len; i++) {
            if (lambda[i] == 0) b[i] = A0;
            else b[i]=modbase(Index_of[lambda[i]] - discr_r + GPRIME - 1);
          }
        } else {
          for (ci = synd_len - 1; ci >= 0; ci--) b[ci + 1] = b[ci];
          b[0] = A0;
        }                        //      COPY(lambda,t,synd_len+1);
        for (ci = synd_len + 1 - 1; ci >= 0; ci--) {
          lambda[ci] = t[ci];
        }
      }
    }      /* Convert lambda to index form and compute deg(lambda(x)) */
    deg_lambda = 0;
    for (i = 0; i < synd_len + 1; i++) {
      lambda[i] = Index_of[lambda[i]];
      if (lambda[i] != A0) deg_lambda = i;
    }
    /* Find roots of the error+erasure locator polynomial by Chien Search*/
    for (ci = synd_len - 1; ci >= 0; ci--) reg[ci + 1] = lambda[ci + 1];
    count = 0;                  /* Number of roots of lambda(x) */
    for (i = 1, k = data_len - 1; i <= GPRIME; i++) {
      q = 1;
      for (j = deg_lambda; j > 0; j--) {
        if (reg[j] != A0) {
          reg[j] = modbase(reg[j] + j);  //  q=modbase(q+Alpha_to[reg[j]]);
          if (deg_lambda != 1) {
            if (j % 2 == 0)  q = (q + Alpha_to[reg[j]]) % GPRIME;
            else    q = (q + GPRIME - Alpha_to[reg[j]]) % GPRIME;
          } else {  q = Alpha_to[reg[j]] % GPRIME; if (q == 1) q--; }
        }
      }
      if (q == 0) {   /* store root (index-form) and error location number */
        root[count] = i;
        loc[count] = GPRIME - 1 - i;
        if (count < synd_len) count += 1;
        else printf("Error : Error count too big = %d \n", count);
      }
      if (k == 0)  k = data_len - 1; else   k -= 1;
      /* If we've already found max possible roots,
       * abort the search to save time       */
      if (count == deg_lambda) break;
    }
    if (deg_lambda != count) {
    /* deg(lambda) unequal to number of roots => uncorrectable error detected*/
      printf("Uncorrectable error: root count = %d deg lambda = %d \n",
               count, deg_lambda);
      count = -1;
      goto finish;
    }
    /*
     * Compute err+eras evaluator poly omega(x) = s(x)*lambda(x) (modulo
     * x**(synd_len)). in index form. Also find deg(omega).     */
    deg_omega = 0;
    for (i = 0; i < synd_len; i++) {
      tmp = 0;
      j = (deg_lambda < i) ? deg_lambda : i;
      for (; j >= 0; j--) {
        if ((s[i + 1 - j] != A0) && (lambda[j] != A0)) {
          if (j % 2 == 1) {
              tmp = (tmp + GPRIME - Alpha_to[modbase(s[i + 1 - j]
                       + lambda[j])]) % GPRIME;
          } else {
              tmp = (tmp + Alpha_to[modbase(s[i + 1 - j] + lambda[j])])
                       % GPRIME;
          }
        }
      }
      if (tmp != 0) deg_omega = i;
      omega[i] = Index_of[tmp];
    }
    omega[synd_len] = A0;
    /*
     * Compute error values in poly-form. num1 = omega(inv(X(l))), num2 =
     * inv(X(l))**(B0-1) and den = lambda_pr(inv(X(l))) all in poly-form
     */
    for (j = count - 1; j >= 0; j--) {
      num1 = 0;
      for (i = deg_omega; i >= 0; i--) {
        if (omega[i] != A0) {
          num1 = (num1 + Alpha_to[modbase(omega[i] + ((i + 1) * root[j]))]) % GPRIME;
        }
      }          //  num2 = Alpha_to[modbase(root[j] * (1 - 1) + data_len)];
      num2 = 1;
      den = 0;
      // denominator if product of all (1 - Bj Bk) for k != j
      // if count = 1, then den = 1
      den = 1;
      for (k = 0; k < count; k += 1) {
        if (k != j) {
          tmp = (1 + GPRIME - Alpha_to[modbase(GPRIME - 1 - root[k]
			  + root[j])]) % GPRIME;
          den = Alpha_to[modbase(Index_of[den] + Index_of[tmp])];
        }
      }
      if (den == 0) {
        printf("\n ERROR: denominator = 0\n");      /* Convert to dual- basis */
        count = -1;
        goto finish;
      }
      error_val = Alpha_to[modbase(Index_of[num1] + Index_of[num2] +
                                   GPRIME - 1 - Index_of[den])] % GPRIME;
      /* Apply error to data */
      if (num1 != 0) {
        if (loc[j] < data_len + 1) {
          fix_loc = data_len - loc[j];
          if (fix_loc < data_len + 1)
              data[fix_loc] = (data[fix_loc] + GPRIME - error_val) % GPRIME;
        }
      }
    }
finish:
  if(eras_pos!=NULL)for(i=0;i<count;i++)if(eras_pos!=NULL)eras_pos[i]=loc[i];
  return count;
}                                       /* Return -1 if wrong cluster */

static int bestham(int word, int which)
{
  int i, best = which;
  if (dump) printf("%d %.4x 0x%08x (%d)\n",
            which, word, dham[which][word], dham[which][word] & 0xffff);
  for (i = 0; i < 3; ++i)
    if ((dham[i][word] & 0xff000000) < (dham[best][word] & 0xff000000))best=i;
  if (best != which) return -1;
  return dham[which][word];
}

static void add_codeword(int word)
{
  static int len = 0;
  static int sorow = 0;                            /* start of row */
  static int skip = 0;

  if (skip) {  skip--;  return;    }
  if (numouts == 0)
      len = word;         /* not really used, but stored as codewords[0] */
  if ((word & 0xffffff) == 0x030000) {                 /* start sequence */
             //if ((word & 0xff00000) == 0x2000000) {  /* start sequence */
    skip = 1;
    sorow = numouts;
    return;
  }
  if ((word & 0xffffff) == 0x030001) {                 /* stop sequence? */
    numouts--;
    return;
  }
  if (word == -3) {                        /* Rewind to beginning of row */
    numouts = sorow;
    return;
  }
  if ((word & 0xffff) == 0xffff) {
    word = 0;
    erasures[numerasures++] = numouts;
  }                                               /* store codeword */
  codewords[numouts++] = word & 0xffff;
}

/*
 *  This routine does all the decoding. Individual compaction segments
 *  are extracted from the codeword array, then for each segment the
 *  decode_segment() function is called, which in turn invokes one of
 *  the convert_*() functions which finally decode the data into a
 *  human readable format.
 */

static void decode_codewords()
{
  int   i, cw, len, slen, mode, shift;
  Int32 segment[34*90];  /* single compaction segment to be decoded */
  if (numouts == 0) return;
  len = codewords[0];
  if (len == 0) return;
  slen = 0;
  mode = 900;    /* default mode is Text Compaction */
  shift = mode;
  for (i = 1; i < len; ++i) {
    cw = codewords[i];
    if (cw >= 900) {
      if (slen > 0) decode_segment(segment, slen, mode);
      slen = 0;
      switch (cw) {
      case 900:  /* mode latch to Text Compaction mode */
      case 901:  /* mode latch to Byte Compaction */
      case 902:  /* mode latch to Numeric Compaction */
          mode = shift = cw;
          break;
      case 913:  /* mode shift to Byte Compaction */
          shift = cw;
          break;
      case 921:  /* reader initialization */
          break;
      case 922:  /* terminator codeword for Macro PDF control block */
          break;
      case 923:  /* Begin of optional fields in the Macro PDF control block */
          break;
      case 924:  /* mode latch to Byte Compaction (num of encoded bytes
                    is an integer multiple of 6) */
          mode = shift = cw;
          break;
      case 925:  /* identifier for a user defined Extended Channel
                    Interpretation (ECI) */
      case 926:  /* identifier for a general purpose ECI format */
      case 927:  /* identifier for an ECI of a character set or code page */
          break;
      case 928:  /* Begin of a Macro PDF Control Block */
          break;
      default:
          fprintf(stderr, "Unknown mode %d\n", cw);
          break;
      }
      continue;
    }
    segment[slen++] = cw;
    if (shift != mode) {
      if (slen > 0) decode_segment(segment, slen, shift);
      slen = 0;
      shift = mode;
    }
  }
  if (slen > 0) decode_segment(segment, slen, mode);
}

void decode_segment(int *cw, int len, int mode)
{
  switch (mode) {
  case 900:
      convert_text(cw, len);
      break;
  case 901:
  case 913:
  case 924:
      convert_byte(cw, len, mode);
      break;
  case 902:
      convert_num(cw, len);
      break;
  }
}

void convert_byte(int *cw, int len, int mode)
{
  UInt64 codeval;
  int i, j;
  unsigned char b[6];
  if (encfmt) printf("BC \"");
                          /* 6 bytes are encoded in a group of 5 codewords */
  for ( ; (mode == 901) ? (len > 5) : (len >= 5); len -= 5) {
    codeval = 0;                      /* convert from base 900 to base 256 */
    for (i = 0; i < 5; ++i) { codeval *= 900; codeval += *cw++;    }
    for (j = 0; j < 6; ++j) { b[5-j] = codeval % 256; codeval >>= 8;  }
    if (encfmt) for (j = 0; j < 6; ++j) printf("%02X", b[j]);
    else        for (j = 0; j < 6; ++j) printf("%c", b[j]);
  }
  /* remaining codewords, if any, are encoded 1 byte per codeword */
  if (len > 0) {
    for (j = 0; j < len; ++j)  b[j] = *cw++;
    if (encfmt) for (j = 0; j < len; ++j) printf("%02X", b[j]);
    else        for (j = 0; j < len; ++j) printf("%c", b[j]);
    codeval = 0;
    i = 0;
  }
  if (encfmt) printf("\"\n");
  fflush(stdout);
}

void convert_text(int *cw, int len)
{
  int mode, shift, enc;
  int i, j, c[2], cout;

  static char txt_upper[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ    ";
  static char txt_lower[] = "abcdefghijklmnopqrstuvwxyz    ";
  static char txt_mixed[] = "0123456789&\r\t,:#-.$/+%*=^     ";
  static char txt_punct[] = ";<>@[\\]_`~!\r\t,:\n-.$/\"|*()?{}' ";

  mode = shift = 0;
  if (encfmt) printf("TC \"");
  for (i = 0; i < len; ++i) {
    c[0] = *cw / 30;
    c[1] = *cw % 30;
    ++cw;
    for (j = 0; j < 2; ++j) {
      enc = mode;
      if (mode != shift) { enc = shift; shift = mode; }
      switch (enc) {
      case 0:  /* uppercase */
          if (c[j] == 27) { mode = shift = 1; continue; }  /* lower latch */
          if (c[j] == 28) { mode = shift = 2; continue; }  /* mixed latch */
          if (c[j] == 29) { shift = 3; continue; }         /* punct shift */
          cout = txt_upper[c[j]];
          break;
      case 1:  /* lowercase */
          if (c[j] == 27) { shift = 0; continue; }         /* upper shift */
          if (c[j] == 28) { mode = shift = 2; continue; }  /* mixed latch */
          if (c[j] == 29) { shift = 3; continue; }         /* punct shift */
          cout = txt_lower[c[j]];
          break;
      case 2:  /* mixed (numeric) */
          if (c[j] == 25) { mode = shift = 3; continue; }  /* punct latch */
          if (c[j] == 27) { mode = shift = 1; continue; }  /* lower latch */
          if (c[j] == 28) { mode = shift = 0; continue; }  /* upper latch */
          if (c[j] == 29) { shift = 3; continue; }         /* punct shift */
          cout = txt_mixed[c[j]];
          break;
      case 3:  /* punctuation */
          if (c[j] == 29) { mode = shift = 0; continue; }  /* upper latch */
          cout = txt_punct[c[j]];
          break;
      }
      printf("%c", cout);
    }
  }
  if (encfmt) printf("\"\n");
  fflush(stdout);
}

#ifndef MIN
#define MIN(x,y) (((x) < (y)) ? (x) : (y))
#endif
/*  BCD arithmetic is used in this routine in order to simplify
    large-precision number manipulations.  */

void convert_num(int *cw, int len)
{
  int n_bcd[45], cw_bcd[3];
  int i, j, n, res, carry, start;

  for ( ; len > 0; len -= 15) {              /* clear the accumulator */
    for (i = 0; i < 45; ++i) n_bcd[i] = 0;
    for (i = 0; i < MIN(len, 15); ++i) {     /* convert codeword to BCD */
      n = *cw++;    cw_bcd[0] = n % 10;
      n /= 10;      cw_bcd[1] = n % 10;
      n /= 10;      cw_bcd[2] = n;
                               /* multiply accumulator by 900 (100 * 9) */
      if (i > 0) {
        carry = 0;              /* multiply by 9 */
        for (j = 0; j < 45; ++j) {
          res = n_bcd[j] * 9 + carry;
          n_bcd[j] = res % 10;
          carry = res / 10;
        }                      /* multiply by 100 */
        for (j = 44; j >= 2; --j)  n_bcd[j] = n_bcd[j-2];
        n_bcd[1] = 0;
        n_bcd[0] = 0;
      }                         /* then add the BCD codeword */
      carry = 0;
      for (j = 0; j < 3; ++j) {
        res = n_bcd[j] + cw_bcd[j] + carry;
        n_bcd[j] = res % 10;
        carry = res / 10;
      }
      for ( ; j < 45; ++j) {
        res = n_bcd[j] + carry;
        n_bcd[j] = res % 10;
        carry = res / 10;
      }
    }
    start = 0;
    if (encfmt) printf("NC \"");
    for (j = 0; j < 45; ++j) {
      if (start)   printf("%c", n_bcd[44-j] + '0');
      else if (n_bcd[44-j] == 1)         start = 1;
      else if (n_bcd[44-j] != 0) {  printf("<invalid>");  break; }
    }
    if (encfmt) printf("\"\n");
  }
}

/* this routine extracts the codewords from a single pixel row from the image */
int processrow(int cols, int rownum, int num, double *cumbits)
{
  int firstblack = 0;
  int scale;
  int j;
  int nchange;                        //int cumchange[cols];
  int cumchange[2048];
  double thresh = 0.5 * num;
  while ((firstblack < cols) && (cumbits[firstblack] < thresh)) ++firstblack;
  if (firstblack+1 >= cols) return 0;
  nchange = 0;
  cumchange[nchange++] = 0;
  for (j = firstblack+1; j < cols; ++j) {
    if ((cumbits[j] < thresh) != (cumbits[j-1] < thresh)) {
      if(nchange>1 &&(j-firstblack-cumchange[nchange-1])*15 < cumchange[1])
          --nchange;                        /* Spurious change */
      else cumchange[nchange++] = j - firstblack;
    }
  }
  if (nchange < 8) return 0;
  for (j = 0; j < nchange-8; j += 8) {
    int k;
    int word = 0;
    unsigned cw;
    scale = cumchange[j+8] - cumchange[j];
    for (k = 0; k < 8; k += 2) {
      int s, e, l;
      s=(int)(17.0*(double)(cumchange[j+k] - cumchange[j])/(double)scale+0.5);
      e=(int)(17.0*(double)(cumchange[j+k+1]-cumchange[j])/(double)scale+0.5);
                           /* We know we always start with 1, end with 0 */
      if (s < 1) s = 1;
      if (e > 16) e = 16;
      for (l = s; l < e; ++l) word |= mask[l-1];
    }
    word >>= 1;
    cw = bestham(word, rownum % 3);
    add_codeword(cw);
  }
  return 1;
}

#define FUZZ_THRESH (cols/40)
#define ROW_THRESH  (cols/20)
bit *bits[100000];
int main()
{
  int rows,cols, i,j,d, ready, num,rownum, nx,ny;
  double *cumbits;
  for (i = 0; i < 15; ++i) mask[i] = 1 << (15-i);

  CxImage *image=new CxImage("3.jpg", CXIMAGE_FORMAT_JPG);
  
  if (!image->IsValid()){ delete image; return 0; }
  cols=(short)(image->GetWidth());
  rows=(short)(image->GetHeight());
printf("clos,rows: %d %d\n",cols,rows);
for(i=0;i<rows;i++){ bits[i]=new bit[cols];  printf("%d ",bits[i]); }
printf("\n");
  ny=rows;  nx=cols;
  for(j=0;j<ny;j++) for(i=0;i<nx;i++)
    bits[ny-1-j][i]=(image->GetPixelGray(i,j)<179)?1:0;
  if(image) delete image;
  
  cumbits = new double[cols];
  ready = 1;  num = rownum = 0;
  for (j = 0; j < cols; j++) cumbits[j] = 0.0;
  for (i = 1; i < rows; i++) {
    for (j=d=0; j < cols; j++) if (bits[i][j] != bits[i-1][j])d++;
    if (d < FUZZ_THRESH) {
      if(ready==1){ for(j=num=0;j<cols;j++)cumbits[j]=0.;  ready=2; }
      for(j=0;j<cols;j++)if(bits[i][j]==PBM_BLACK)cumbits[j]+=1.;
      num++;
    } else if(d > ROW_THRESH) {
      if (ready == 2) {
        if (processrow(cols, rownum, num, cumbits)) rownum++;
        ready = 1;
      }
    }
  }
  if(ready==2)if(processrow(cols,rownum,num,cumbits))rownum++;
  decode_codewords();  return 0;
}
